/*
 * @Author: your name
 * @Date: 2022-01-06 09:45:17
 * @LastEditTime: 2022-04-14 16:15:03
 * @LastEditors: your name
 * @Description: axios封装
 */
import axios from 'axios'
import { MessageBox, Message } from 'element-ui'
import store from '@/store'
import { getToken, getUserId, getCodeToken } from '@/utils/auth'
import { parseTime } from '@/utils'
import { Loading } from 'element-ui'
// create an axios instance
const service = axios.create({
  //baseURL: baseURL, // url = base url + request url
  // baseURL: 'http://192.168.3.239:8998',
  baseURL: `${PLATFROM_CONFIG.baseUrl}`,
  // withCredentials: true, // send cookies when cross-domain requests
  timeout: 30000, // request timeout
  headers: {
    'Content-Type': 'application/json'
  }
})
let loadingInstance,
  flag = false
// request interceptor
service.interceptors.request.use(
  config => {
    if (config.istoken) {
      config.headers['appSecret'] = PLATFROM_CONFIG.appSecret
    } else if (getCodeToken() && !config.istoken && config.islogin) {
      config.headers['token'] = getCodeToken()
    } else if (store.getters.token) {
      // let each request carry token
      // ['X-Token'] is a custom headers key
      // please modify it according to the actual situation
      config.headers['Token'] = getToken()
    }
    if (config.data && !config.url.includes('/api/fs4a/sendLog')) {
      config.data['OperatorId'] = getUserId()
    }
    flag = true
    if (flag) {
      loadingInstance = Loading.service({
        text: '数据加载中'
      })
    }
    if (config.url.includes('/api/fs4a/sendLog') && config.data.Data[0]?.OperateResult?.Token) {
      config.headers['token'] = config.data.Data[0].OperateResult.Token
    }
    config.metaData = {
      BeginTime: parseTime(new Date(), '{y}-{m}-{d} {h}:{i}:{s}')
    }
    return config
  },
  error => {
    // do something with request error
    console.log(error) // for debug
    return Promise.reject(error)
  }
)

// response interceptor
service.interceptors.response.use(
  /**
   * If you want to get http information such as headers or status
   * Please return  response => response
   */

  /**
   * Determine the request status by custom code
   * Here is just an example
   * You can also judge the status by HTTP Status Code
   */
  response => {
    const res = response.data
    // 特定请求完成后发送日志
    const { metaData, url, Ident, data, msg, islogin } = response.config
    if (!url.includes('/api/fs4a/sendLog') && (Ident || msg)) {
      Object.assign(metaData, {
        LogInfo: typeof data === 'string' ? JSON.parse(data) : data,
        EndTime: parseTime(new Date(), '{y}-{m}-{d} {h}:{i}:{s}'),
        LogType: msg ?? '未定义操作',
        OperateResult: res.returnData,
        Ident: Ident ?? ''
      })
      const { BeginTime, EndTime, OperateResult } = metaData
      const config = {
        url: '/api/fs4a/sendLog',
        method: 'post',
        data: {
          Data: [metaData],
          BrowserTag: navigator.userAgent,
          OperateIP: returnCitySN.cip,
          Msg: msg,
          Type: '2',
          BeginTime,
          EndTime,
          OperateResult
        }
      }
      if (islogin) {
        if (res.returnData) {
          config.data.OperatorId = config.data.Operator = res.returnData.UserId.toString()
        }
      } else {
        config.data.OperatorId = config.data.Operator = getUserId()
      }
      service(config)
    }
    if (res.code == 0 || res.code == -1) {
      flag = false
      loadingInstance.close()
      return res
    } else {
      if (res.code == 500) {
        Message({
          message: '身份令牌过期或失效，即将重新登录',
          type: 'error',
          duration: 5 * 1000,
          onClose: () => {
            store.dispatch('tagsView/delAllViews').then(() => {})
            store.dispatch('user/resetToken').then(() => {
              location.reload()
            })
          }
        })
      }
      flag = false
      loadingInstance.close()
    }
    // if the custom code is not 20000, it is judged as an error.
    // if (res.code != 0 || res.code == -1) {
    //   Message({
    //     message: res.message || 'Error',
    //     type: 'error',
    //     duration: 5 * 1000
    //   })

    //   // 50008: Illegal token; 50012: Other clients logged in; 50014: Token expired;
    //   if (res.code == 500) {
    //     // to re-login
    //     Message({
    //       message: '身份令牌过期或失效，即将重新登录',
    //       type: 'error',
    //       duration: 5 * 1000,
    //       onClose: () => {
    //         store.dispatch('tagsView/delAllViews').then(() => { })
    //         store.dispatch('user/resetToken').then(() => {
    //           location.reload()
    //         })
    //       }
    //     })
    //   }
    //   // loadingInstance.close()
    //   return Promise.reject(new Error(res.message || 'Error'))
    // } else {
    //   // loadingInstance.close()
    //   return res
    // }
  },
  error => {
    // console.log('err', error) // for debug
    const des = `${error}`.split(' ').includes('500')
    if (des) {
      Message({
        message: '身份令牌过期或失效，即将重新登录',
        type: 'error',
        duration: 5 * 1000,
        onClose: () => {
          store.dispatch('app/toggleDialog', false)
          store.dispatch('tagsView/delAllViews').then(() => {})
          store.dispatch('user/resetToken').then(() => {
            location.reload()
          })
        }
      })
      // MessageBox.confirm('You have been logged out, you can cancel to stay on this page, or log in again', 'Confirm logout', {
      //   confirmButtonText: 'Re-Login',
      //   cancelButtonText: 'Cancel',
      //   type: 'warning'
      // }).then(() => {
      //   store.dispatch('user/resetToken').then(() => {
      //     location.reload()
      //   })
      // })
    } else {
      Message({
        message: error.message,
        type: 'error',
        duration: 5 * 1000
      })
    }
    flag = false
    loadingInstance.close()
    return Promise.reject(error)
  }
)

export default service
